@@ -1,8 +1,8 @@
-/* Output or logging functions for GNU Make.  
+/* Output or logging functions for GNU Make.
 
 Copyright (C) 2005, 2007, 2008 R. Bernstein <rocky@gnu.org>
 This file is part of GNU Make (remake variant).
-Copyright (C) 2004, 2005, 2007, 2008, Free Software Foundation, Inc.  
+Copyright (C) 2004, 2005, 2007, 2008, Free Software Foundation, Inc.
 
 GNU Make is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@ along with GNU Make; see the file COPYING.  If not, write to
 the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 Boston, MA 02111-1307, USA.  */
 
+#include "makeint.h"
 #include "make.h"
 #include "main.h"
 #include "commands.h"
@@ -28,136 +29,96 @@ Boston, MA 02111-1307, USA.  */
 #include "read.h"
 #include "print.h"
 
+#include <stdarg.h>
+
 /* Think of the below not as an enumeration but as #defines done in a
    way that we'll be able to use the value in a gdb. */
 enum debug_print_enums_e debug_print_enums1;
 
-#if HAVE_VPRINTF || HAVE_DOPRNT
-# define HAVE_STDVARARGS 1
-# if __STDC__
-#  include <stdarg.h>
-#  define VA_START(args, lastarg) va_start(args, lastarg)
-# else
-#  include <varargs.h>
-#  define VA_START(args, lastarg) va_start(args)
-# endif
-# if HAVE_VPRINTF
-#  define VA_PRINTF(fp, lastarg, args) vfprintf((fp), (lastarg), (args))
-# else
-#  define VA_PRINTF(fp, lastarg, args) _doprnt((lastarg), (args), (fp))
-# endif
-# define VA_END(args) va_end(args)
-#else
-/* # undef HAVE_STDVARARGS */
-# define va_alist a1, a2, a3, a4, a5, a6, a7, a8
-# define va_dcl char *a1, *a2, *a3, *a4, *a5, *a6, *a7, *a8;
-# define VA_START(args, lastarg)
-# define VA_PRINTF(fp, lastarg, args) fprintf((fp), (lastarg), va_alist)
-# define VA_END(args)
-#endif
-
 
-/* Print a message on stdout.  */
+/* Write a message indicating what directory we are in
+   and the make level number.  */
 
-void
-#if __STDC__ && HAVE_STDVARARGS
-message (int prefix, const char *fmt, ...)
-#else
-message (int prefix, const char *fmt, va_alist)
-#endif
+static void
+err_log_working_directory (void)
 {
-#if HAVE_STDVARARGS
-  va_list args;
-#endif
+  static char *buf = NULL;
+  static unsigned int len = 0;
+  unsigned int need;
+  const char *fmt;
+  char *p;
 
-  log_working_directory (1);
+  /* Get enough space for the longest possible output.  */
+  need = strlen (program) + INTSTR_LENGTH + 2 + 1;
+  if (starting_directory)
+    need += strlen (starting_directory);
 
-  if (fmt != 0)
+  /* Use entire sentences to give the translators a fighting chance.  */
+  if (makelevel == 0)
+    if (starting_directory == 0)
+        fmt = _("%s\n");
+    else
+        fmt = _("%s: directory '%s'\n");
+  else
+    if (starting_directory == 0)
+        fmt = _("%s[%u]: unknown directory\n");
+    else
+        fmt = _("%s[%u]: directory '%s'\n");
+
+  need += strlen (fmt);
+
+  if (need > len)
     {
-      if (prefix)
-	{
-	  if (makelevel == 0)
-	    printf ("%s: ", program);
-	  else
-	    printf ("%s[%u]: ", program, makelevel);
-	}
-      VA_START (args, fmt);
-      VA_PRINTF (stdout, fmt, args);
-      VA_END (args);
-      putchar ('\n');
+      buf = xrealloc (buf, need);
+      len = need;
     }
 
-  fflush (stdout);
-}
+  p = buf;
+  if (print_data_base_flag)
+    {
+      *(p++) = '#';
+      *(p++) = ' ';
+    }
 
-/* Print an error message.  */
-
-void
-#if HAVE_ANSI_COMPILER && USE_VARIADIC && HAVE_STDARG_H
-error (const floc_t *flocp, const char *fmt, ...)
-#else
-error (flocp, fmt, va_alist)
-     const floc_t *flocp;
-     const char *fmt;
-     va_dcl
-#endif
-{
-#if USE_VARIADIC
-  va_list args;
-#endif
-
-  log_working_directory (1);
-
-  if (flocp && flocp->filenm)
-    fprintf (stderr, "%s:%lu: ", flocp->filenm, flocp->lineno);
-  else if (makelevel == 0)
-    fprintf (stderr, "%s: ", program);
+  if (makelevel == 0)
+    if (starting_directory == 0)
+      sprintf (p, fmt , program);
+    else
+      sprintf (p, fmt, program, starting_directory);
+  else if (starting_directory == 0)
+    sprintf (p, fmt, program, makelevel);
   else
-    fprintf (stderr, "%s[%u]: ", program, makelevel);
+    sprintf (p, fmt, program, makelevel, starting_directory);
 
-  VA_START(args, fmt);
-  VA_PRINTF (stderr, fmt, args);
-  VA_END (args);
-
-  putc ('\n', stderr);
-  fflush (stderr);
-  if (debugger_on_error & DEBUGGER_ON_ERROR) 
-    enter_debugger(NULL, NULL, -1, DEBUG_ERROR_HIT);
+  outputs (0, buf);
 }
 
+
 void
-#if __STDC__ && HAVE_STDVARARGS
 err_with_stack (target_stack_node_t *p_call, const char *fmt, ...)
-#else
-err_with_stack (p_call, fmt, va_alist)
-     target_stack_node_t *p_call;
-     const char *fmt;
-     va_dcl
-#endif
 {
-#if HAVE_STDVARARGS
   va_list args;
-#endif
-  floc_t *p_floc   = NULL;
+  gmk_floc *p_floc   = NULL;
   file_t *p_target = NULL;
-   
-  log_working_directory (1);
 
   if (p_call && p_call->p_target) {
     p_target = p_call->p_target;
     p_floc   = &(p_target->floc);
   }
-  
+
   if (p_floc && p_floc->filenm)
     fprintf (stderr, "%s:%lu: ", p_floc->filenm, p_floc->lineno);
-  else if (makelevel == 0)
-    fprintf (stderr, "%s: ", program);
-  else
-    fprintf (stderr, "%s[%u]: ", program, makelevel);
+  else {
+    err_log_working_directory ();
+    if (makelevel == 0)
+      fprintf (stderr, "%s: ", program);
+    else
+      fprintf (stderr, "%s[%u]: ", program, makelevel);
+  }
 
-  VA_START(args, fmt);
-  VA_PRINTF (stderr, fmt, args);
-  VA_END (args);
+  va_start (args, fmt);
+  vfprintf (stderr, fmt, args);
+  va_end (args);
 
   putc ('\n', stderr);
   if (!no_extended_errors) {
@@ -171,86 +132,25 @@ err_with_stack (p_call, fmt, va_alist)
   }
   fflush (stdout);
   fflush (stderr);
-  if (debugger_on_error & DEBUGGER_ON_ERROR) 
+  if (debugger_on_error & DEBUGGER_ON_ERROR)
     enter_debugger(p_call, p_target, -1, DEBUG_ERROR_HIT);
 }
 
 /* Print an error message and exit.  */
-
 void
-#if HAVE_ANSI_COMPILER && USE_VARIADIC && HAVE_STDARG_H
-fatal (const floc_t *flocp, const char *fmt, ...)
-#else
-fatal (flocp, fmt, va_alist)
-     const floc_t *flocp;
-     const char *fmt;
-     va_dcl
-#endif
+fatal_err (target_stack_node_t *p_call, const char *fmt, ...)
 {
-#if USE_VARIADIC
   va_list args;
-#endif
-
-  log_working_directory (1);
-
-  if (flocp && flocp->filenm)
-    fprintf (stderr, "%s:%lu: *** ", flocp->filenm, flocp->lineno);
-  else if (makelevel == 0)
-    fprintf (stderr, "%s: *** ", program);
-  else
-    fprintf (stderr, "%s[%u]: *** ", program, makelevel);
-
-  VA_START(args, fmt);
-  VA_PRINTF (stderr, fmt, args);
-  VA_END (args);
-
-  fputs (_(".  Stop.\n"), stderr);
-
-  /* If in_debugger == 1 and we don't die or enter a the debugger again.
-     if in_debugger == 77 then we just propagate that wish.
-     
-   */
-  switch (in_debugger) {
-  case 0:
-    if ( (debugger_on_error & DEBUGGER_ON_FATAL) || debugger_enabled )
-      enter_debugger(NULL, NULL, 2, DEBUG_ERROR_HIT);
-    die (2);
-    break;
-  case DEBUGGER_QUIT_RC:
-    die(DEBUGGER_QUIT_RC);
-  default:
-  case 1: 
-    longjmp(debugger_loop, 0);
-    break;
-    
-  }
-}
-
-/* Print an error message and exit.  */
-
-void
-#if __STDC__ && HAVE_STDVARARGS
-fatal_err(target_stack_node_t *p_call, const char *fmt, ...)
-#else
-fatal_err (flocp, fmt, va_alist)
-     target_stack_node_t *p_call;
-     const char *fmt;
-     va_dcl
-#endif
-{
-#if HAVE_STDVARARGS
-  va_list args;
-#endif
-  floc_t *p_floc   = NULL;
+  gmk_floc *p_floc   = NULL;
   file_t *p_target = NULL;
 
-  log_working_directory (1);
+  err_log_working_directory ();
 
   if (p_call && p_call->p_target) {
     p_target = p_call->p_target;
     p_floc   = &(p_target->floc);
   }
-  
+
   if (p_floc && p_floc->filenm)
     fprintf (stderr, "%s:%lu: *** ", p_floc->filenm, p_floc->lineno);
   else if (makelevel == 0)
@@ -258,20 +158,20 @@ fatal_err (flocp, fmt, va_alist)
   else
     fprintf (stderr, "%s[%u]: *** ", program, makelevel);
 
-  VA_START(args, fmt);
-  VA_PRINTF (stderr, fmt, args);
-  VA_END (args);
+  va_start (args, fmt);
+  vfprintf (stderr, fmt, args);
+  va_end (args);
 
   fputs (_(".  Stop.\n"), stderr);
   if (!no_extended_errors) {
-    if (p_call) 
+    if (p_call)
       print_target_stack(p_call, -1, MAX_STACK_SHOW);
     else if (p_stack_floc_top)
       print_floc_stack(-1, MAX_STACK_SHOW);
   }
   if ( (debugger_on_error & DEBUGGER_ON_FATAL) || debugger_enabled )
     enter_debugger(p_call, p_target, 2, DEBUG_ERROR_HIT);
-  die (2);
+  die (MAKE_FAILURE);
 }
 
 #ifndef HAVE_STRERROR
@@ -295,15 +195,6 @@ strerror (int errnum)
 }
 #endif
 
-/*! Print an error message from errno.  */
-void
-perror_with_name (const char *str, const char *name)
-{
-  error (NILF, _("%s%s: %s"), str, name, strerror (errno));
-  if (debugger_on_error & DEBUGGER_ON_ERROR) 
-    enter_debugger(NULL, NULL, -1, DEBUG_ERROR_HIT);
-}
-
 /*! Under -d, write a message describing the current IDs.  */
 
 void
@@ -323,13 +214,13 @@ log_access (char *flavor)
 }
 
 /*! Display a variable and its value. */
-void 
+void
 print_variable (variable_t *p_v)
 {
   if (p_v) {
     const char *psz_origin = origin2str(p_v->origin);
     if (NULL != p_v->fileinfo.filenm) {
-      printf(_("%s:%lu (origin: %s) %s = %s\n"), 
+      printf(_("%s:%lu (origin: %s) %s = %s\n"),
 	     p_v->fileinfo.filenm, p_v->fileinfo.lineno,
 	     psz_origin,
 	     p_v->name, p_v->value);
@@ -340,26 +231,26 @@ print_variable (variable_t *p_v)
 }
 
 /*! Display a variable and its value with all substitutions included. */
-void 
+void
 print_variable_expand (variable_t *p_v)
 {
   if (p_v) {
     const char *psz_origin = origin2str(p_v->origin);
     if (NULL != p_v->fileinfo.filenm) {
-      printf(_("%s:%lu (origin: %s) %s := %s\n"), 
+      printf(_("%s:%lu (origin: %s) %s := %s\n"),
 	     p_v->fileinfo.filenm, p_v->fileinfo.lineno,
 	     psz_origin,
 	     p_v->name, variable_expand(p_v->value));
     } else {
-      printf("(origin %s) %s := %s\n", psz_origin, 
+      printf("(origin %s) %s := %s\n", psz_origin,
 	     p_v->name, variable_expand(p_v->value));
     }
   }
 }
 
 /*! Show a command before executing it. */
-extern void 
-print_target_prefix (const char *p_name) 
+extern void
+print_target_prefix (const char *p_name)
 {
   printf(" %s", p_name);
   if (makelevel != 0) {
@@ -368,21 +259,21 @@ print_target_prefix (const char *p_name)
 }
 
 /*! Show target information: location and name. */
-extern void 
-print_file_target_prefix (const file_t *p_target) 
+extern void
+print_file_target_prefix (const file_t *p_target)
 {
   print_floc_prefix(&(p_target->floc));
   print_target_prefix(p_target->name);
 }
 
 /*! Show a command before executing it. */
-extern void 
-print_floc_prefix (const floc_t *p_floc) 
+extern void
+print_floc_prefix (const gmk_floc *p_floc)
 {
   if (!p_floc) return;
   if (p_floc->filenm) {
-    if (!basename_filenames && strlen(p_floc->filenm) 
-	&& p_floc->filenm[0] != '/') 
+    if (!basename_filenames && strlen(p_floc->filenm)
+	&& p_floc->filenm[0] != '/')
       printf("%s/", starting_directory);
     printf("%s:%lu", p_floc->filenm, p_floc->lineno);
   } else {
@@ -405,11 +296,11 @@ print_child_cmd (child_t *p_child, target_stack_node_t *p)
     debug_enter_reason_t reason = DEBUG_STEP_HIT;
     if (i_debugger_stepping)
       reason = DEBUG_STEP_HIT;
-    else if (p_child->file->tracing & BRK_BEFORE_PREREQ) 
+    else if (p_child->file->tracing & BRK_BEFORE_PREREQ)
       reason = DEBUG_BRKPT_BEFORE_PREREQ;
-    else if (p_child->file->tracing & BRK_BEFORE_PREREQ) 
+    else if (p_child->file->tracing & BRK_BEFORE_PREREQ)
       reason = DEBUG_BRKPT_AFTER_PREREQ;
-      
+
     rc=enter_debugger(p, p_child->file, 0, reason);
   }
 
@@ -417,20 +308,20 @@ print_child_cmd (child_t *p_child, target_stack_node_t *p)
 }
 
 void
-print_target_stack_entry (const file_t *p_target, int i, int i_pos) 
+print_target_stack_entry (const file_t *p_target, int i, int i_pos)
 {
-  floc_t floc;
-  const char *psz_target_name = 
+  gmk_floc floc;
+  const char *psz_target_name =
     (p_target && p_target->name) ? p_target->name : "(null)";
-  
+
   /* If we don't have a line recorded for the target,
      but we do have one for the commands it runs,
      use that.
   */
   if (p_target->floc.filenm) {
-    memcpy(&floc, &(p_target->floc), sizeof(floc_t));
+    memcpy(&floc, &(p_target->floc), sizeof(gmk_floc));
   } else if (p_target->cmds) {
-    memcpy(&floc, &(p_target->cmds->fileinfo.filenm), sizeof(floc_t));
+    memcpy(&floc, &(p_target->cmds->fileinfo.filenm), sizeof(gmk_floc));
     /* HACK: is it okay to assume that the target is on the line
        before the first command? Or should we list the line
        that the command starts on - so we know we've faked the location?
@@ -439,22 +330,22 @@ print_target_stack_entry (const file_t *p_target, int i, int i_pos)
   } else {
     floc.filenm = NULL;
   }
-  
+
   if (floc.filenm) {
     if (i_pos != -1) {
       printf("%s", (i == i_pos) ? "=>" : "  ");
     }
-    printf ("#%u  %s at ", i, psz_target_name);
+    printf ("#%d  %s at ", i, psz_target_name);
     print_floc_prefix(&floc);
   } else {
     if (i_pos != -1) {
       printf("%s", (i == i_pos) ? "=>" : "  ");
     }
     if (p_target->phony)
-      printf ("#%u  %s (.PHONY target)", i, psz_target_name);
-    else 
-      printf ("#%u  %s at ??", i, psz_target_name);
-    
+      printf ("#%d  %s (.PHONY target)", i, psz_target_name);
+    else
+      printf ("#%d  %s at ??", i, psz_target_name);
+
   }
   printf ("\n");
 }
@@ -463,11 +354,11 @@ print_target_stack_entry (const file_t *p_target, int i, int i_pos)
 /*! Display the target stack. i_pos is the position we are currently.
   i_max is the maximum number of entries to show.
  */
-extern void 
+extern void
 print_target_stack (target_stack_node_t *p, int i_pos, int i_max)
 {
   int i=0;
-  for ( ; p && i < i_max ; 
+  for ( ; p && i < i_max ;
 	i++, p = p->p_parent  ) {
     print_target_stack_entry (p->p_target, i, i_pos);
   }
@@ -475,18 +366,18 @@ print_target_stack (target_stack_node_t *p, int i_pos, int i_max)
 
 /*! Display the Makefile read stack. i_pos is the position we are currently.
   i_max is the maximum number of entries to show. */
-extern void 
+extern void
 print_floc_stack (int i_pos, int i_max)
 {
   int i=0;
   floc_stack_node_t *p;
   printf("\n");
-  for ( p=p_stack_floc_top; p && i < i_max ; 
+  for ( p=p_stack_floc_top; p && i < i_max ;
 	i++, p = p->p_parent ) {
     if (i_pos != -1) {
       printf("%s", (i == i_pos) ? "=>" : "  ");
     }
-    printf ("#%u  ", i);
+    printf ("#%d  ", i);
     if (p->p_floc->filenm) {
       print_floc_prefix(p->p_floc);
     }
@@ -495,7 +386,7 @@ print_floc_stack (int i_pos, int i_max)
 }
 
 /*! Print the file information.  */
-void print_file (file_t *p_file) 
+void print_file (file_t *p_file)
 {
   char buf[FILE_TIMESTAMP_PRINT_LEN_BOUND + 1];
   printf("File %s:\n", p_file->name);
@@ -532,7 +423,7 @@ bool print_read_makefiles(const char *psz_filename)
 }
 
 /*! Print the command line used to invoke Make. */
-void print_cmdline (void) 
+void print_cmdline (void)
 {
   unsigned int i;
   printf(_("Command-line invocation:"));
@@ -550,9 +441,9 @@ void print_cmdline (void)
 }
 
 
-/* 
+/*
  * Local variables:
  * eval: (c-set-style "gnu")
  * indent-tabs-mode: nil
- * End: 
+ * End:
  */
